@@ -3,9 +3,9 @@ astLib - python astronomy modules
 
 (c) 2007-2012 Matt Hilton
 
-(c) 2013-2023 Matt Hilton & Steven Boada
+(c) 2013-2024 Matt Hilton & Steven Boada
 
-U{http://astlib.sourceforge.net}
+U{https://astlib.readthedocs.io}
 
 See the README file for information on usage and installation.
 See the LICENSE file for information on distribution.
@@ -13,4 +13,4 @@ See the LICENSE file for information on distribution.
 """
 
 __all__=['astCalc', 'astCoords', 'astImages', 'astPlots', 'astStats', 'astWCS', 'astSED']
-__version__ = '0.11.10'
+__version__ = '0.12.0'
