@@ -9,7 +9,8 @@
 import sys
 import math
 import numpy
-from PyWCSTools import wcscon
+import astropy.coordinates as apyCoords
+#from PyWCSTools import wcscon
 #import IPython
 
 #-----------------------------------------------------------------------------
@@ -346,17 +347,20 @@ def convertCoords(inputSystem, outputSystem, coordX, coordY, epoch):
 
     """
 
-    if inputSystem=="J2000" or inputSystem=="B1950" or inputSystem=="GALACTIC":
-        if outputSystem=="J2000" or outputSystem=="B1950" or \
-            outputSystem=="GALACTIC":
+    # Replaced WCSTools wcscon routine here with astropy
+    # This is only here to save breaking old code that still uses this routine
+    sysMap={'J2000': 'fk5', 'B1950': 'fk4', 'GALACTIC': 'galactic'}
+    if inputSystem not in sysMap.keys() or outputSystem not in sysMap.keys():
+        raise Exception("inputSystem and outputSystem must be 'J2000', 'B1950'\
+            or 'GALACTIC'")
+    c=apyCoords.SkyCoord(coordX, coordY, frame = sysMap[inputSystem], unit='deg')
+    b=c.transform_to(sysMap[outputSystem])
+    if outputSystem in ['J2000', 'B1950']:
+        outCoords=[b.ra.deg, b.dec.deg]
+    elif outputSystem == 'GALACTIC':
+        outCoords=[b.l.deg, b.b.deg]
 
-            outCoords=wcscon.wcscon(wcscon.wcscsys(inputSystem),
-                wcscon.wcscsys(outputSystem), 0, 0, float(coordX), float(coordY), epoch)
-
-            return outCoords
-
-    raise Exception("inputSystem and outputSystem must be 'J2000', 'B1950'"
-                    "or 'GALACTIC'")
+    return outCoords
 
 #-----------------------------------------------------------------------------
 def calcRADecSearchBox(RADeg, decDeg, radiusSkyDeg):
